@@ -598,6 +598,17 @@ namespace CecSharp
 			return gcnew String(retVal);
 		}
 
+		String ^ GetLibInfo()
+		{
+			const char *retVal = m_libCec->GetLibInfo();
+			return gcnew String(retVal);
+		}
+
+		void InitVideoStandalone()
+		{
+			m_libCec->InitVideoStandalone();
+		}
+
 	private:
 		ICECAdapter *        m_libCec;
     CecCallbackMethods ^ m_callbacks;
